/**
 * @file ntmcmd.h
 * @author Shinichiro Nakamura
 * @brief コマンド解釈層の定義。
 */

/*
 * ===============================================================
 *  Natural Tiny Monitor (NT-Monitor)
 * ===============================================================
 * Copyright (c) 2011-2012 Shinichiro Nakamura
 * Inspired by M, Murakami
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * ===============================================================
 */

#ifndef NTMCMD_H
#define NTMCMD_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
    int (*serial_read)(char *buf, int cnt, void *extobj);
    int (*serial_write)(const char *buf, int cnt, void *extobj);
    void *user_extobj;
} ntmcmd_extobj_t;

#define NTMCMD_USER_EXTOBJ(P)               ((P)->user_extobj)
#define NTMCMD_SERIAL_READ(P, BUF, CNT)     ((P)->serial_read(BUF, CNT, NTMCMD_USER_EXTOBJ(P)))
#define NTMCMD_SERIAL_WRITE(P, BUF, CNT)    ((P)->serial_write(BUF, CNT, NTMCMD_USER_EXTOBJ(P)))

/**
 * @brief コマンドを実行する。
 * @details ntshellモジュールのコールバックという位置付けである。
 *
 * @param s ユーザから渡って来たコマンド文字列。
 * @param extobj 外部オブジェクト。
 *
 * @return ntshellモジュールに返す値。
 */
int ntmcmd_execute(const char *s, void *extobj);

#ifdef __cplusplus
}
#endif

#endif

